@@ -70,52 +70,48 @@ Rcpp::RawVector serialize_cpp_obj(T *model_outputs)
         oarchive(*model_outputs);
     }
     ss.seekg(0, ss.end);
-    Rcpp::RawVector retval(ss.tellg());
+    /* Checking for potential integer overflows */
+    std::stringstream::pos_type vec_size = ss.tellg();
+    if (vec_size <= 0) {
+        Rcpp::Rcerr << "Error: model is too big to serialize, resulting object will not be usable.\n" << std::endl;
+        return Rcpp::RawVector();
+    }
+    Rcpp::RawVector retval((size_t) vec_size);
     ss.seekg(0, ss.beg);
     ss.read(reinterpret_cast<char*>(&retval[0]), retval.size());
     return retval;
 }
 
-// [[Rcpp::export]]
-SEXP deserialize_IsoForest(Rcpp::RawVector src)
+template <class T>
+SEXP deserialize_cpp_obj(Rcpp::RawVector src)
 {
     std::stringstream ss;
     ss.write(reinterpret_cast<char*>(&src[0]), src.size());
     ss.seekg(0, ss.beg);
-    std::unique_ptr<IsoForest> model_outputs = std::unique_ptr<IsoForest>(new IsoForest());
+    std::unique_ptr<T> model_outputs = std::unique_ptr<T>(new T());
     {
         cereal::BinaryInputArchive iarchive(ss);
         iarchive(*model_outputs);
     }
-    return Rcpp::XPtr<IsoForest>(model_outputs.release(), true);
+    return Rcpp::XPtr<T>(model_outputs.release(), true);
+}
+
+// [[Rcpp::export]]
+SEXP deserialize_IsoForest(Rcpp::RawVector src)
+{
+    return deserialize_cpp_obj<IsoForest>(src);
 }
 
 // [[Rcpp::export]]
 SEXP deserialize_ExtIsoForest(Rcpp::RawVector src)
 {
-    std::stringstream ss;
-    ss.write(reinterpret_cast<char*>(&src[0]), src.size());
-    ss.seekg(0, ss.beg);
-    std::unique_ptr<ExtIsoForest> model_outputs = std::unique_ptr<ExtIsoForest>(new ExtIsoForest());
-    {
-        cereal::BinaryInputArchive iarchive(ss);
-        iarchive(*model_outputs);
-    }
-    return Rcpp::XPtr<ExtIsoForest>(model_outputs.release(), true);
+    return deserialize_cpp_obj<ExtIsoForest>(src);
 }
 
 // [[Rcpp::export]]
 SEXP deserialize_Imputer(Rcpp::RawVector src)
 {
-    std::stringstream ss;
-    ss.write(reinterpret_cast<char*>(&src[0]), src.size());
-    ss.seekg(0, ss.beg);
-    std::unique_ptr<Imputer> imputer = std::unique_ptr<Imputer>(new Imputer());
-    {
-        cereal::BinaryInputArchive iarchive(ss);
-        iarchive(*imputer);
-    }
-    return Rcpp::XPtr<Imputer>(imputer.release(), true);
+    return deserialize_cpp_obj<Imputer>(src);
 }
 
 // [[Rcpp::export]]
@@ -140,7 +136,7 @@ double* set_R_nan_as_C_nan(double *x, size_t n, int nthreads)
     for (size_t_for i = 0; i < n; i++)
         if (isnan(x[i]))
             x[i] = NAN;
-    return &x[0];
+    return x;
 }
 
 // [[Rcpp::export]]
@@ -518,7 +514,7 @@ void predict_iso(SEXP model_R_ptr, Rcpp::NumericVector outp, Rcpp::IntegerVector
 
     if (X_cat.size())
     {
-        categ_data_ptr   =  &X_cat[0];
+        categ_data_ptr    =  &X_cat[0];
     }
 
     if (Xc.size())
@@ -540,7 +536,7 @@ void predict_iso(SEXP model_R_ptr, Rcpp::NumericVector outp, Rcpp::IntegerVector
         tree_num_ptr = &tree_num[0];
     }
 
-    double* depths_ptr    =  &outp[0];
+    double* depths_ptr =  &outp[0];
 
     IsoForest*     model_ptr      =  NULL;
     ExtIsoForest*  ext_model_ptr  =  NULL;
@@ -682,7 +678,7 @@ Rcpp::List impute_iso(SEXP model_R_ptr, SEXP imputer_R_ptr, bool is_extended,
                           *imputer_ptr);
 
     return Rcpp::List::create(
-                Rcpp::_["X_num"] = Xr.size()? Xr : X_num,
+                Rcpp::_["X_num"] = (Xr.size())? (Xr) : (X_num),
                 Rcpp::_["X_cat"] = X_cat
             );
 }
