@@ -421,9 +421,10 @@ int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
     /* TODO: find a better way of handling interrupt signals when calling in Python/R.
        The following will still change the behavior of interrupts when called through e.g. Flask */
     #if !defined(_WIN32) && !defined(_WIN64) && !defined(_MSC_VER)
-    struct sigaction sig_handle;
+    struct sigaction sig_handle = {};
     sig_handle.sa_flags = SA_RESETHAND;
     sig_handle.sa_handler = set_interrup_global_variable;
+    sigemptyset(&sig_handle.sa_mask);
     #endif
 
     /* grow trees */
